diff --git a/lib/utils/json.c b/lib/utils/json.c
index 2041db757e9..33bc66bb789 100644
--- a/lib/utils/json.c
+++ b/lib/utils/json.c
@@ -8,6 +8,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
+#include <math.h>
 #include <zephyr/sys/printk.h>
 #include <zephyr/sys/util.h>
 #include <stdbool.h>
@@ -980,8 +981,67 @@ static int num_encode(const int32_t *num, json_append_bytes_t append_bytes,
 static int float_ascii_encode(struct json_obj_token *num, json_append_bytes_t append_bytes,
 		      void *data)
 {
+	float f;
+	memcpy(&f, &num->start, sizeof(f));
 
-	return append_bytes(num->start, num->length, data);
+	/* max size decimal scientfic notation
+	 *	 +/- 1.2345678 e +/- xy
+	 */
+	char buf[1 + 1 + 1 + 7 + 1 + 1 + 2 + 1];
+	char* p = buf;
+
+	/* special case zero */
+	if (f == 0.0f) {
+		*p++ = '0';
+	} else {
+		if (f < 0.0f) {
+			f *= -1.0f;
+			*p++ = '-';
+		}
+
+		if (f == 1.0f) {
+			*p++ = '1';
+		} else {
+			/* setup to have a non-zero ones digit */
+			float e = log10f(f);
+			if (e < 0.0f)
+				f *= powf(10.0f, ceilf(-e));
+			else if (e > 1.0f)
+				f /= powf(10.0f, floorf(e));
+
+			/* mantissa */
+			float whole, frac;
+			frac = modff(f, &whole);
+			*p++ = '0' + whole;
+			*p++ = '.';
+			f = frac * 10.0f;
+			for (unsigned i = 0; i < 7 && f > 0.01f; i++) {
+				frac = modff(f, &whole);
+				*p++ = '0' + whole;
+				f = frac * 10.0f;
+			}
+
+			/* exponent */
+			*p++ = 'e';
+			if (e < 0.0f) {
+				*p++ = '-';
+				e = ceilf(-e);
+			}
+			whole = e;
+			e = floor(log10f(e));
+			do {
+				float d = powf(10.0f, e);
+				float q = floorf(whole / d);
+				float r = whole - (q * d);
+				*p++ = '0' + q;
+				e -= 1.0f;
+				whole = r;
+			} while (e > 1.0f);
+		}
+	}
+	size_t len = p - buf;
+	*p++ = '\0';
+	return append_bytes(buf, len, data);
 }
 
 static int opaque_string_encode(struct json_obj_token *opaque, json_append_bytes_t append_bytes,
